{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}delete-query-modal{% endblock %}
{% block modal_title %}Delete Query{% endblock %}

{% block modal_body %}
Are you sure you want to delete the query?
{% endblock %}

{% block modal_footer %}
<a class="btn btn-default pull-left" data-dismiss="modal" href="#"><i class="fa fa-times"></i> Cancel</a>
<a id="btn-delete-query" class="btn btn-default">Confirm</a>
{% endblock %}