<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>Query ended</title>
</head>
<body>
Dear {{firstname}} {{lastname}} ,
<br><br>

We have the pleasure to let you know that your query {{query_name}} has been successfully ended.<br>
Please login to the website to see the result files.

<br><br>

Thank you,
<br><br>
</body>
</html>
